#include <stdint.h>

// Sets a real variable from an integer.
void ti_real_set(uint8_t* variable, int16_t value) {
	for (uint8_t i = 0; i < 9; ++i) variable[i] = 0x00;
	uint16_t abs_value = value;
	if (value < 0) {
		abs_value = -value;
		variable[0] |= 0x80;
	}
	variable[1] = 0x80;
	uint8_t parts[6] = { 0, 0, 0, 0, 0, 0 };
	uint16_t factor = 10000;
	for (uint8_t part_offset = 0; part_offset < 5; ++part_offset) {
		while (abs_value >= factor) {
			++parts[part_offset];
			abs_value -= factor;
		}
		factor /= 10;
	}
	if (value != 0) {
		variable[1] = 0x84;
		while (parts[0] == 0) {
			--variable[1];
			for (uint8_t part_index = 0; part_index < 5; ++part_index) {
				parts[part_index] = parts[part_index + 1];
			}
		}
	}
	for (uint8_t part_index = 0; part_index < 6; ++part_index) {
		if (part_index & 1) {
			variable[(part_index / 2) + 2] |= parts[part_index];
		} else {
			variable[(part_index / 2) + 2] = parts[part_index] << 4;
		}
	}
}

// Gets an integer value from a real variable. Destroys the original variable.
int16_t ti_real_get(uint8_t* variable) {
	int16_t value = 0;
	if (!(variable[0] & 0x80)) {
		
		int8_t exponent = 13 - (variable[1] - 0x80);
		if (exponent < 0) {
			value = 0;
		} else {
			while (exponent--) {
				for (int8_t p = 6; p >= 0; --p) {
					variable[p + 2] = (variable[p + 2] >> 4) | ((variable[p + 1] & 15) << 4);
				}
				variable[2] &= 0x0F;
			}
			for (uint8_t p = 0; p < 7; ++p) {
				value *= 10;
				value += variable[p + 2] >> 4;
				value *= 10;
				value += variable[p + 2] & 15;
			}
		}
		
		if (variable[0] & 0x80) {
			value = -value;
		}
	}
	return value;
}