#ifndef TILP_H
#define TILP_H

#define TILP_PORT  PORTB // PORT register that the link port is connected to.
#define TILP_DDR    DDRB // DDR register that the link port is connected to.
#define TILP_PIN    PINB // PIN register that the link port is connected to.

#define TILP_RED       1 // Bit index within the port the red wire of the link cable is connected to.
#define TILP_WHITE     2 // Bit index within the port the white wire of the link cable is connected to.

// Drive a line low.
#define TILP_LINE_LOW(line) TILP_PORT &= (uint8_t)~_BV(line);TILP_DDR |= _BV(line)

// Float the line.
#define TILP_LINE_HIGH(line) TILP_DDR &= (uint8_t)~_BV(line);TILP_PORT |= _BV(line)

// Sample a line.
#define TILP_LINE(line) (TILP_PIN & _BV(line))

// Time out counter value (in 13.11ms units).
#define TILP_TIME_OUT_TIME (76)

enum tilp_result_t {
	TILP_OK,
	TILP_TIME_OUT,
	TILP_NOT_ACKNOWLEDGED,
	TILP_BAD_CHECKSUM,
};

// Initialises the TI link protocol routines.
void tilp_init(void);

// Write a single byte using the TI link protocol.
enum tilp_result_t tilp_write_byte(uint8_t value);

// Read a single byte using the TI link protocol.
enum tilp_result_t tilp_read_byte(uint8_t* value);

// Write a block of values using the TI link protocol.
enum tilp_result_t tilp_write_block(uint8_t* block, uint16_t block_length);

// Write a block of values and a checksum using the TI link protocol.
enum tilp_result_t tilp_write_block_and_checksum(uint8_t* block, uint16_t block_length);

// Write a block of values from ROM using the TI link protocol.
enum tilp_result_t tilp_write_block_P(PGM_VOID_P block, uint16_t block_length);

// Read a block of values using the TI link protocol.
enum tilp_result_t tilp_read_block(uint8_t* block, uint16_t block_length);

// Read a block of values and a checksum using the TI link protocol.
enum tilp_result_t tilp_read_block_and_checksum(uint8_t* block, uint16_t block_length);

// Writes an acknowledgement.
enum tilp_result_t tilp_write_acknowledgement(uint8_t sender);

// Reads an acknowledgement.
enum tilp_result_t tilp_read_acknowledgement(uint8_t* sender);

#endif