#include <avr/io.h>

#include "video.h"

// A 96x64 pixel video buffer.
uint8_t video_buffer[768] = {
	#include "initial_image.h"
};

// Stores the current video standard (read-only).
enum video_standard_t video_standard;

// Initialise the assembly driver for PAL.
void video_driver_init_pal(void);

// Initialise the assembly driver for NTSC.
void video_driver_init_ntsc(void);

// Stores the X offset for the PAL signal. Must be between VIDEO_PAL_X_OFFSET_MIN and VIDEO_PAL_X_OFFSET_MAX inclusive.
uint8_t video_pal_x_offset = 28;

// Stores the Y offset for the PAL signal. Must be between VIDEO_PAL_Y_OFFSET_MIN and VIDEO_PAL_Y_OFFSET_MAX inclusive.
int16_t video_pal_y_offset = -32;

// Stores the X offset for the NTSC signal. Must be between VIDEO_NTSC_X_OFFSET_MIN and VIDEO_NTSC_X_OFFSET_MAX inclusive.
uint8_t video_ntsc_x_offset = 34;

// Stores the Y offset for the NTSC signal. Must be between VIDEO_NTSC_Y_OFFSET_MIN and VIDEO_NTSC_Y_OFFSET_MAX inclusive.
int16_t video_ntsc_y_offset = -38;

// Initialise the video signal generator.
void video_init(enum video_standard_t standard) {
	
	// Initialise the assembly driver.
	switch (video_standard = standard) {
		case VIDEO_STANDARD_PAL:
			video_driver_init_pal();
			break;
		default:
			video_driver_init_ntsc();
			break;
	}

	// Set up I/O pins.
	VIDEO_PICTURE_DDR |= _BV(VIDEO_PICTURE_BIT);
	VIDEO_SYNC_DDR |= _BV(VIDEO_SYNC_BIT);

	// Set up timer.
	TCCR1B |= (0b001 << CS10); // Run at native clock speed, no prescaling.
	TIMSK1 |= _BV(OCIE1A);     // Interrupt on match A.
	TCCR1B |= _BV(WGM12);      // Mode 4 (TOP=OCR1A, CTC).
	
}