#ifndef VIDEO_H
#define VIDEO_H

#define VIDEO_PICTURE_PORT       PORTD   // Port that the picture data is output from.
#define VIDEO_PICTURE_DDR         DDRD   // Data direction register for the port that the picture data is output from.
#define VIDEO_PICTURE_BIT            7   // Bit index representing the pin in the port that the picture signal is output from (must be 7).

#define VIDEO_SYNC_PORT          PORTB   // Port that the sync signal is output from.
#define VIDEO_SYNC_DDR            DDRB   // Data direction register for the port that the sync signal is output from.
#define VIDEO_SYNC_BIT               0   // Bit index representing the pin in the port that the sync signal is output from.

#ifdef __ASSEMBLER__

#define r_sreg_save r2

#else  /* !ASSEMBLER */

#include <stdint.h>

enum video_standard_t {
	VIDEO_STANDARD_NTSC,
	VIDEO_STANDARD_PAL,
};

// Stores the current video standard (read-only).
extern enum video_standard_t video_standard;

#define VIDEO_PAL_X_OFFSET_MIN 1
#define VIDEO_PAL_X_OFFSET_MAX 42

// Stores the X offset for the PAL signal. Must be between VIDEO_PAL_X_OFFSET_MIN and VIDEO_PAL_X_OFFSET_MAX inclusive.
extern uint8_t video_pal_x_offset;

#define VIDEO_PAL_Y_OFFSET_MIN -49
#define VIDEO_PAL_Y_OFFSET_MAX 0

// Stores the Y offset for the PAL signal. Must be between VIDEO_PAL_Y_OFFSET_MIN and VIDEO_PAL_Y_OFFSET_MAX inclusive.
extern int16_t video_pal_y_offset;

#define VIDEO_NTSC_X_OFFSET_MIN 1
#define VIDEO_NTSC_X_OFFSET_MAX 58

// Stores the X offset for the NTSC signal. Must be between VIDEO_NTSC_X_OFFSET_MIN and VIDEO_NTSC_X_OFFSET_MAX inclusive.
extern uint8_t video_ntsc_x_offset;

#define VIDEO_NTSC_Y_OFFSET_MIN -60
#define VIDEO_NTSC_Y_OFFSET_MAX 0

// Stores the Y offset for the NTSC signal. Must be between VIDEO_NTSC_Y_OFFSET_MIN and VIDEO_NTSC_Y_OFFSET_MAX inclusive.
extern int16_t video_ntsc_y_offset;

register uint8_t r_sreg_save asm("r2");

// A 96x64 pixel video buffer.
extern uint8_t video_buffer[768];

// Initialise the video signal generator.
void video_init(enum video_standard_t video_standard);

#endif /* ASSEMBLER */

#endif